#!/bin/ksh
#
# File: apply_pending.sh
#
#   This script is used to apply SQL scripts listed in the PENDING.txt file in
# a database's docs directory. If a PENDING.txt file does not exists in the database's directory
# and it is not the production database then it looks in the Quality database if one exists and
# then the Test database. This allows for Development and Test databases to share a PENDING.txt
# file with the Quality database. Scripts will be run in the order which they appear
# in the PENDING.txt file, and must have a .sql extension name. If a script is required to run
# as a schema owner then the script can be prefixed with the schema owner name, and a colon. No
# spaces. In 11g and above you also need to supply the schema owner's password in the format:
# OwnerName/Password:ScriptName
#
#   You can also specify a dirctory other than the docs directory that contains the scripts to be
# applied, however the docs directories will be searched in the order of the Current Database,
# Quality (if found), Test and Development. The fist directory found that contains all of the
# scripts listed in the PENDING.txt is the directory that will be used. The log files generated
# from running each script will have the same name, but a .log extension and it will be written
# to in the docs directory for the database where they were applied.
#
#   An email notice will be sent to the members of sid_email.txt as well as any email address
# specified in the clone flag file if apply_pending.sh was called from a clone script. The email
# will contain the contents of the PENDING.txt file that was used as well as the log files. The 
# PENDING.txt file will also be updated with comments that indicate the last time scripts were 
# applied.
#
#   The developers can modify the PENDING.txt list by submitting a revised list inside of
# the flag file which is transferred to the host. The flag file can also specify to have no 
# pending scripts applied. To provided this functionality, copy the developers with the script
# in the /oracle_share/docs/Oracle_FlagRefresh directory. Specifically FlagRefresh_DATABASE.vbs.txt,
# AbortRefresh_DATABASE.vbs.txt, and FlagRefresh_README.txt
#
#   When applying script to test or quality databases where other scripts have already been applied,
# you can append new scripts to the PENDING.txt file an specify the "-new" parameter on the end of the
# call to the apply_pending.sh script. This will cause the apply_pending.sh script to skip any 
# script in the pending list that has "--Last Applied" on the same line.
#  
#   When scripts are applied to production, they will be comment out from the PENDING.txt list in 
# the production database docs directory as well as PENDING.txt list for the Quality, Test, and 
# Development directories if they exist.
#
# Before apply_pending.sh applies any scripts it runs a hot snapshot backup with a 3 day retention.
#
# To deploy:
# Replace the current clone_snap_???_2_???.sh file with clone_snap_SOURCE_2_TARGET.sh script but preserve
# the configuration settings as well and any post clone SQL scripting.
#
# Author: Doug Jackson  3/12/2009.

#--------------------------------------------------------------------
#- Clean Up
#--------------------------------------------------------------------
proc_cleanup()
{
  # Check $LOG for ERROR and send mail if it exist or if VERBOSE=YES
  if [ ! -z "`grep ^ERROR: $LOG`" ]
  then
    SUBJECT="ERROR: Failed to apply Pending SQL scripts to $SID"
  else
    if [ ! -z "`grep WARNING: $LOG`" ]
    then
      SUBJECT="Applied Pending SQL scripts to $SID with WARNINGS."
    else
      SUBJECT="Applied Pending SQL scripts to $SID with No Warnings."
    fi
  fi
  >$TEMPFILE1
  # Send to the user that trigger the apply_pending_on_demand.sh 
  FLAG_FILE=${FLAG_DIR}/flag_apply_pending_`echo $SID | tr [A-Z] [a-z]`
  CALLED_BY=`grep USER_NAME= ${FLAG_FILE}* | cut -d= -f2`  
  if [ ! -z "$CALLED_BY" ]
  then
    echo Apply Pending Called on Demand by: $CALLED_BY >>$LOG
    if [ -z "`echo $CALLED_BY | grep -i OKE03910`" ]
    then 
      # Eliminate Dublicate emails to Doug Jackson
      echo ${CALLED_BY}@oneok.com >>$TEMPFILE1
    fi
  fi
  # Send to recipients for ALL sids
  cat $EMAIL_LIST | grep -v "^#" | grep "^ALL:" | cut -d: -f2 >>$TEMPFILE1
  # Send to recipients for current sid
  cat $EMAIL_LIST | grep -v "^#" | grep "^$SID:" | cut -d: -f2 >>$TEMPFILE1
  # If this was applied to a schema then we pull the contact from the CAPP schema.
  if [ ! -z "$SCHEMA" ]
  then
USER_ID_MAIL_LIST=`sqlplus -s ${CAPP_CONNECTION} << ENDIT
set pause off head off pagesize 0 feedback off linesize 200
select distinct emp_id||'@oneok.com' from capp.capp_contacts_vw where schema = '$SCHEMA';
exit
ENDIT`
    for USER_ID_MAIL in `echo $USER_ID_MAIL_LIST`
    do
      echo $USER_ID_MAIL >>$TEMPFILE1
    done
  fi 
  # Send to recipients in the clone flag file if there is one
  if [ -r "$FLAG_FILE" ] 
  then
    grep @ $FLAG_FILE | grep -v ^# >>$TEMPFILE1
  fi  

  echo "" >>$LOG
  echo "Notified:" >>$LOG  
  for EACH in `cat $TEMPFILE1  | sort | uniq `
  do
    echo  $EACH >>$LOG
  done

  for EACH in `cat $TEMPFILE1  | sort | uniq `
  do
    mail -s "$SUBJECT" $EACH < $LOG
  done

  rm -f $TEMPFILE1 2>/dev/null
  rm -f $TEMPFILE2 2>/dev/null
}

#################################################################################################################
### MAIN PROGRAM
#################################################################################################################

SCRIPT_DIR=`dirname $0`
if [ "$SCRIPT_DIR" = "." ]
then
  SCRIPT_DIR=`pwd`
fi
cd $SCRIPT_DIR

HOST=`hostname`
HOST=`echo $HOST | sed 's/.oneok.com//g'`

#Configuration
EMAIL_LIST="$SCRIPT_DIR/sid_email.txt"
FLAG_DIR="/u01/flag_ftp"
REQUIRE_SCHEMA="DVLP:TEST:PROD DEV10:TST10:PRD10 DEV11:TST11:PRD11"
ORATAB=/etc/oratab
DOCS=/oracle_share/docs
CAPP_CONNECTION=capp/topthis@prod
RESTRICT_HOST_COMMAND=YES
TEMPFILE1=$SCRIPT_DIR/apply_pending_1_$$.tmp
TEMPFILE2=$SCRIPT_DIR/apply_pending_2_$$.tmp

if [ ! -f $EMAIL_LIST ]
then
  echo "ERROR: $EMAIL_LIST DOES NOT EXIST! apply_pending.sh failed!!!"
  mail -s "ERROR: $EMAIL_LIST DOES NOT EXIST! apply_pending.sh failed!" djackson@oneok.com < /dev/null
  exit 1
fi

if [ $# -lt 1 ]
then
  echo "Usage:  apply_pending.sh <Target_DB> [Schema] [-new] [-sb]"
  echo "  Target_DB - Database where scripts listed in /docs/HOST_Target_DB/PENDING.txt will be applied."
  echo "  The Target_DB must be on current Host."
  echo "  Source Directory Defaults, in order to: HOST_Target_DB, Quality, Test, Development."
  echo "  -new = Only scripts in the pending list without \"-- Last Applied\" will be applied."
  echo "  -sb = Skip backup."
  echo "  The header of script files can contain shell commands to be run on the host. Prefix lines with:"
  echo "  -- HOST COMMAND:"
  exit 0
fi

# Check for oratab file existence
if [ ! -f $ORATAB ]
then
  echo "ERROR: $ORATAB DOES NOT EXIST! - apply_pending.sh failed!"
  exit 1
fi

SID=`echo $1 | tr [a-z] [A-Z]`

if [ ! -z "`echo $REQUIRE_SCHEMA | grep $SID`" ]
then
  # A schema is required
  if [ -z "$2" ]
  then
    echo "ERROR: The $SID database contains multiple schemas. A schema name must be specified"
    echo "Usage:  apply_pending.sh <Target_DB> [Schema]"
    exit 1
  fi
  SCHEMA=`echo $2 | tr [a-z] [A-Z]`
  LOG=$SCRIPT_DIR/apply_pending_${SID}_${SCHEMA}.log
else
  SCHEMA=""
  LOG=$SCRIPT_DIR/apply_pending_${SID}.log
fi

if [ -z "`echo \"$* x\" | grep -i \"\-n\"`" ]
then
  ONLY_NEW=NO
else
  ONLY_NEW=YES
fi

if [ -z "`echo \"$* x\" | grep -i \"\-sb\"`" ]
then
  SKIP_BACKUP=NO
else
  SKIP_BACKUP=YES
fi

echo "Applying Pending Scripts to database: $SID $SCHEMA  `date`"
echo "Applying Pending Scripts to database: $SID $SCHEMA  `date`" >$LOG

# Verify this SID is in the ORATAB
if [ -z "`grep ^${SID}: $ORATAB`" ]
then
  echo "ERROR: SID: $SID not found in oratab."
  echo "ERROR: SID: $SID not found in oratab." >> $LOG
  proc_cleanup
  exit 1
fi

# Verify the SID docs Directory
if [ ! -z "$SCHEMA" ]
then
  SID_DIR=$DOCS/`echo $HOST | tr [a-z] [A-Z]`_${SID}_${SCHEMA}
else
  SID_DIR=$DOCS/`echo $HOST | tr [a-z] [A-Z]`_${SID}
fi
if [ ! -r ${SID_DIR} ]
then
  echo "ERROR: Docs Directory: $SID_DIR was not found or is not readable."
  echo "ERROR: Docs Directory: $SID_DIR was not found or is not readable." >> $LOG
  proc_cleanup
  exit 1
fi

# Check if this is a production host
if [ ! -z "`echo $HOST | grep -i prod`" ]
then
  PRODUCTION="YES"
  PROD_SID=$SID
else
  PRODUCTION="NO"
  # Trim off the fist character to get the production database name
  PROD_SID=`echo $SID | cut -c 2-`
fi 
if [ ! -z "$SCHEMA" ]
then
  for SET in `echo $REQUIRE_SCHEMA`
  do
    if [ ! -z "`echo $SET | grep $SID`" ]
    then
      PROD_SID=`echo $SET | cut -d: -f3`
      TEST_SID=`echo $SET | cut -d: -f2`
      DVLP_SID=`echo $SET | cut -d: -f1`
    fi
  done
fi 

# Check if there is a PENDING_IN_PROGRESS.txt file
if [ -f "${SID_DIR}/PENDING_IN_PROGRESS.txt" ]
then
  echo "ERROR: Aborting because a PENDING_IN_PROGRESS.txt file was found in $SID_DIR."
  echo "ERROR: Aborting because a PENDING_IN_PROGRESS.txt file was found in $SID_DIR." >> $LOG
  proc_cleanup
  exit 1
fi

# Check if there is a PENDING.txt file
PENDING=`ls ${SID_DIR} | grep -i ^PENDING.txt$`
if [ -z "$PENDING" ]
then
  # No PENDING.txt found
  # Check if this is the production or quality database.
  if [ "$PRODUCTION" = "YES" ] || [ "$SID" = "Q${PROD_SID}" ]
  then
    echo "No PENDING.txt file was found in $SID_DIR."
    echo "No PENDING.txt file was found in $SID_DIR." >> $LOG
    proc_cleanup
    exit 1
  else
    # This is not production or quality so we check the Quality directory for a pending file, if one exist.
    DIR=`ls $DOCS | grep \_Q${PROD_SID}$` 
    if [ ! -z "$DIR" ] && [ -d "$DOCS/$DIR" ]
    then
      PENDING=`ls $DOCS/$DIR | grep -i ^PENDING.txt$`
      if [ ! -z "$PENDING" ]
      then
        echo "Using a copy of the Quality Database $PENDING in $DOCS/$DIR"    
        echo "Using a copy of the Quality Database $PENDING in $DOCS/$DIR" >> $LOG   
        cp $DOCS/$DIR/$PENDING $SID_DIR/QUAILTY_PENDING.txt
        PENDING=QUAILTY_PENDING.txt
      fi 
    else
      # Try the Test database.
      if [ ! -z "$SCHEMA" ]
      then
        DIR=`ls $DOCS | grep \_${TEST_SID}_${SCHEMA}$` 
      else
        DIR=`ls $DOCS | grep \_T${PROD_SID}$` 
      fi
      if [ ! -z "$DIR" ] && [ -d "$DOCS/$DIR" ]
      then
        PENDING=`ls $DOCS/$DIR | grep -i ^PENDING.txt$`
        if [ ! -z "$PENDING" ]
        then
          echo "Using a copy of the Test Database $PENDING in $DOCS/$DIR"    
          echo "Using a copy of the Test Database $PENDING in $DOCS/$DIR" >> $LOG   
          cp $DOCS/$DIR/$PENDING $SID_DIR/TEST_PENDING.txt
          PENDING=TEST_PENDING.txt
        else
          echo "No PENDING.txt file was found."
          echo "No PENDING.txt file was found." >> $LOG
          proc_cleanup
          exit 1
        fi 
      fi
    fi
  fi
fi

# Clean the PENDING.txt list of any commented scripts that are either "--Applied to Production" or "--Removed"
# Also strip out any ^M which is entered as Ctrl-V Ctrl-M
> $TEMPFILE1
echo "" >> ${SID_DIR}/$PENDING
cat ${SID_DIR}/$PENDING | tr -d '\r' | tr -d '\015\032'  | while read LINE
do
  if [ -z "`echo $LINE | egrep '(--Applied to Production|--Removed)'`" ]
  then
    if [ ! -z "$LINE" ]
    then
      echo $LINE  >> $TEMPFILE1
    fi
  fi
done 
cat $TEMPFILE1 > ${SID_DIR}/$PENDING
echo "" >> ${SID_DIR}/$PENDING

# Check that all of the scripts have a .sql extension.  This prevents accidentally listing the log files as pending.
if [ ! -z "`grep -i [a-z] $SID_DIR/$PENDING | grep -v ^# | cut -d\" \" -f1 | rev | cut -d. -f1 | rev | grep -v -i SQL`" ]
then
  echo "Scripts in the $PENDING file must have an extension name of .sql"
  echo "Scripts in the $PENDING file must have an extension name of .sql" >> $LOG
  proc_cleanup
  exit 1
fi

# Check if there is a clone running for this SID.
if [ ! -z "`ps -ef | grep clone_snap_ | grep _2_${SID}.sh`" ]
then
  # A clone is running.  So check if there is a flag file 
  LOWER_SID=`echo $SID | tr [A-Z] [a-z]`
  FLAG_FILE=/u01/flag_ftp/flag_ftp_clone_*_2_${LOWER_SID}.txt_in_progress
  if [ -r $FLAG_FILE ]
  then
    echo "Contents of flag file: $FLAG_FILE" >> $LOG
    cat $FLAG_FILE >> $LOG
    echo "" >> $LOG
    # Check if the flag file has "DO NOT APPLY PENDING SCRIPTS" and if so, then no scripts are applied
    if [ ! -z "`grep -i \"^DO NOT APPLY PENDING SCRIPTS\" $FLAG_FILE | grep -v ^#`" ]
    then
      if [ -z "`grep -i [a-z] $SID_DIR/$PENDING | grep -v ^#`" ]
      then
        echo "No scripts are listed in the $SID_DIR/$PENDING file."
        echo "No scripts are listed in the $SID_DIR/$PENDING file." >> $LOG
        echo "The flag file specifies: DO NOT APPLY PENDING SCRIPTS"
        echo "The flag file specifies: DO NOT APPLY PENDING SCRIPTS" >> $LOG
      else
        echo "NOTE: There are Pending Scripts but the flag file specifies: DO NOT APPLY PENDING SCRIPTS"
        echo "NOTE: There are Pending Scripts but the flag file specifies: DO NOT APPLY PENDING SCRIPTS" >> $LOG
        echo "Pending Scripts in $SID_DIR/$PENDING:"
        echo "Pending Scripts in $SID_DIR/$PENDING:" >> $LOG
        cat $SID_DIR/$PENDING | grep -v ^# 
        cat $SID_DIR/$PENDING | grep -v ^# >> $LOG 
      fi
      proc_cleanup
      exit 1
    fi
    # Check the flag file for Pending scripts
    # Also strip out any ^M which is entered Ctrl-V Ctrl-M
    NEW_PENDING=`cat $FLAG_FILE | tr -d '
' | grep -v @ | grep -v ^# | cut -d' ' -f1 | grep -v ^$`
    if [ ! -z "$NEW_PENDING" ]
    then
      # Update the existing PENDING.txt
      # Keep any comments and blank lines
      cat ${SID_DIR}/$PENDING | grep -v ^[A-z] > ${SID_DIR}/PENDING_revised.txt

      CHANGED=NO
      # Step through the NEW_PENDING list looking for scripts being added back to the Pending list.
      for SCRIPT in `echo $NEW_PENDING`
      do
        # Check if the script is new and add it if it is.  This will only work for an old script that is being added back.
        # otherwise the actual script will not exist.
        if [ -z "`grep $SCRIPT ${SID_DIR}/$PENDING | grep -v ^#`" ]
        then
          # New Script or Old script that is being added back to the list.
          echo "$SCRIPT --Restored" >> ${SID_DIR}/PENDING_revised.txt
          CHANGED=YES
        fi
      done
 
      # Step through the PENDING.txt list looking for any scripts that are not in NEW_PENDING and so need to be removed.
      cat $SID_DIR/$PENDING | grep -v ^# | cut -d' ' -f1 | grep -v ^$ | while read SCRIPT
      do
        if [ -z "`echo $NEW_PENDING | grep $SCRIPT`" ]
        then
          # Removed
          echo "# $SCRIPT --Removed" >> ${SID_DIR}/PENDING_revised.txt
          CHANGED=YES
        else
          echo $SCRIPT >> ${SID_DIR}/PENDING_revised.txt
        fi
      done

      if [ "$CHANGED" = "YES" ]
      then
        mv ${SID_DIR}/PENDING_revised.txt ${SID_DIR}/$PENDING
        echo "NOTE: Changes were made to the $PENDING list. See Restored/Removed below:"
        echo "NOTE: Changes were made to the $PENDING list. See Restored/Removed below:" >> $LOG
      else
        rm -f ${SID_DIR}/PENDING_revised.txt
      fi

    fi
  fi
fi

# Check for any scripts in the PENDING.txt file
if [ -z "`grep -i [a-z] $SID_DIR/$PENDING | grep -v ^#`" ]
then
  echo "No scripts are listed in $PENDING."
  echo "No scripts are listed in $PENDING." >> $LOG
  proc_cleanup
  exit 1
fi

##HERE
##HERE
##HERE
##HERE

echo "Using Pending List file: $SID_DIR/$PENDING" 
echo "Using Pending List file: $SID_DIR/$PENDING" >> $LOG
if [ "$ONLY_NEW" = "YES" ]
then
  echo Only New Scripts will be applied. Any script with the comment \"--Last Applied\" will be skipped.
  echo Only New Scripts will be applied. Any script with the comment \"--Last Applied\" will be skipped. >> $LOG
fi
echo ""
echo "" >>$LOG
echo "PENDING SCRIPTS:"
echo "PENDING SCRIPTS:" >> $LOG
cat $SID_DIR/$PENDING | grep -v ^# | grep -v ^$
cat $SID_DIR/$PENDING | grep -v ^# | grep -v ^$ >> $LOG
echo ""
echo "" >>$LOG

# To find the source directory that contains all of the SQL script listed in PENDING.txt we will search 
# in order the SID_DIR,  Quality, Test, and Development directories for this database untill we find one directory
# that contains all of the scripts in PENDING.txt. The SID_DIR is most often be the case, where scripts are being
# applied back to Qual or Test after a clone.

# Build a list of the directories for this database, excluding production.
if [ ! -z "$SCHEMA" ]
then
  LOOK_IN_LIST=`basename $SID_DIR`
  for DB in `echo $TEST_SID $DVLP_SID`
  do
    DIR=`ls $DOCS | grep \_${DB}_${SCHEMA}$`
    if [ -d "$DOCS/$DIR" ]
    then
      if [ ! "`basename $SID_DIR`" = "$DIR" ]
      then
        LOOK_IN_LIST=`echo $LOOK_IN_LIST $DIR`
      fi
    fi
  done
else
  LOOK_IN_LIST=`basename $SID_DIR`
  for PREFIX in `echo Q T D`
  do
    DIR=`ls $DOCS | grep \_${PREFIX}${PROD_SID}$`
    if [ -d "$DOCS/$DIR" ]
    then
      if [ ! "`basename $SID_DIR`" = "$DIR" ]
      then
        LOOK_IN_LIST=`echo $LOOK_IN_LIST $DIR`
      fi
    fi
  done
fi

# Step throught the list looking for a directory that contains all of the scripts.
for SRC_DIR in `echo $LOOK_IN_LIST`
do
  FOUND_ALL="YES"
  FOUND_SOME="NO"
  NOT_FOUND=""
  # Trim off any schema owner prefix from the file. IE: owner:script_name.sql 
  for SCRIPT in `cat $SID_DIR/$PENDING | grep -v ^# | cut -d' ' -f1 | grep -v ^$ | cut -d: -f2`
  do
    DOCS=`echo $DOCS`
    SRC_DIR=`echo $SRC_DIR`
    SCRIPT=`echo $SCRIPT`
    if [ -r "$DOCS/$SRC_DIR/$SCRIPT" ]
    then
      FOUND_SOME=YES
    else
      FOUND_ALL=NO
      NOT_FOUND=`echo $NOT_FOUND $SCRIPT`
    fi
  done
  if [ "$FOUND_ALL" = "YES" ]
  then
    break
  else
    echo "Not Found in $SRC_DIR :"
    echo "Not Found in $SRC_DIR :" >> $LOG
    echo $NOT_FOUND
    echo $NOT_FOUND >> $LOG
  fi
  if [ "$FOUND_SOME" = "YES" ]
  then
    echo "NOTE:  Some of the Pending scripts were found in $SRC_DIR."
    echo "NOTE:  Some of the Pending scripts were found in $SRC_DIR." >> $LOG
  fi 
done

if [ "$FOUND_ALL" = "NO" ]
then
  echo "ERROR: All of the scripts listed in $SID_DIR/$PENDING must be in the same Source Directory."
  echo "ERROR: All of the scripts listed in $SID_DIR/$PENDING must be in the same Source Directory." >> $LOG
  echo "  The following $DOCS directories were searched: $LOOK_IN_LIST" 
  echo "  The following $DOCS directories were searched: $LOOK_IN_LIST" >> $LOG
  proc_cleanup
  exit 1
fi

echo "All of the Pending scripts were found in: $SRC_DIR"
echo "All of the Pending scripts were found in: $SRC_DIR" >> $LOG

if [ "$SKIP_BACKUP" = "NO" ]
then

  # Create a hot snapshot backup with a 3 day retention.
  echo "Starting Hot snapshot backup with 3 day retention." 
  echo "Starting Hot snapshot backup with 3 day retention." >> $LOG

  START_TIME=`date +"%y%m%d%H%M"`
  $SCRIPT_DIR/backup.sh hot $SID -snap=3 -v
  # Verify it by looking for a restore script and compairing the time stamp with the start time.
  LOWER_SID=`echo $SID | tr [A-Z] [a-z]`
  RESTORE_SCRIPT=`ls RESTORE_${SID}_SNAPSHOT_${LOWER_SID}_* | sort | tail -1`
  SCRIPT_TIME=`echo $RESTORE_SCRIPT | awk -F "_${LOWER_SID}_" '{print $2}' | cut -d_ -f1,2 | tr -d _`
  if [ -z "$SCRIPT_TIME" ] || [ "$SCRIPT_TIME" -lt "$START_TIME" ]
  then
    echo "ERROR: Backup of $SID can not be verified. No scripts will be applied."
    echo "ERROR: Backup of $SID can not be verified. No scripts will be applied." >> $LOG
    proc_cleanup
    exit 1
  else
    echo "Hot backup completed. The restore script is: $RESTORE_SCRIPT"
    echo "Hot backup completed. The restore script is: $RESTORE_SCRIPT" >> $LOG
  fi
fi

# Set the environment
echo "Setting oracle environment to $SID..."  
echo "Setting oracle environment to $SID..." >> $LOG  
# Source the Oracle users's profile to set the path
. ~/.profile noup > /dev/null
export ORACLE_SID=$SID
export ORAENV_ASK=NO
. oraenv
. set_lib.sh
. set_base.sh

if [ "$PRODUCTION" = "YES" ]
then
  echo "# Scripts applied on `date`" > $SID_DIR/Latest_Applied.txt
fi 

# Make sure there is a rule that will allow oracle to connect as another user.
sqlplus /nolog << ENDIT >/dev/null
connect / as sysdba
DECLARE
x char(1);
BEGIN
select 'x' into x from auth.auth_control where condition = 'OSUSER like oracle' and description = 'oracle su access';
exception when no_data_found then
insert into auth.auth_control (condition,permission,permission_level,action,description)
  values ('OSUSER like oracle','AUTHORIZED',99,'NONE','oracle su access');
commit;  
END;
/
exit
ENDIT

# Start a PENDING_IN_PROGRESS.txt file which will replace PENDING.txt 
> ${SID_DIR}/PENDING_IN_PROGRESS.txt

# Apply the PENDING scripts
cat $SID_DIR/$PENDING | while read LINE
do
  if [ ! -z "`echo $LINE | grep ^#`" ] || [ -z "$LINE" ]
  then
    # Just a commented or a blank line.
    echo $LINE >> ${SID_DIR}/PENDING_IN_PROGRESS.txt
    continue
  fi

  if [ "$ONLY_NEW" = "YES" ] 
  then
    if [ ! -z "`echo $LINE | grep \"\-\-Last Applied\"`" ]
    then
      # This is a script that has already been applied.
      echo $LINE >> ${SID_DIR}/PENDING_IN_PROGRESS.txt
      continue
    fi
  fi

  # Strip away any trailing text and schema owner.
  OWNER=""
  REAL_PASSWD=""
  SCRIPT=`echo $LINE | cut -d" " -f1 | cut -d: -f2`
  if [ ! -z "`echo $LINE | cut -d\- -f1 | grep :`" ]
  then
    OWNER=`echo $LINE | cut -d: -f1 | cut -d/ -f1`
    STR=`echo $LINE | cut -d: -f1`/
    REAL_PASSWD=`echo $STR | cut -d/ -f2` 
  fi

  echo " " 
  echo " " >> $LOG
  echo "======================================================================================== "
  echo "======================================================================================== " >> $LOG
  echo "======================================================================================== "
  echo "======================================================================================== " >> $LOG
  if [ ! -z "$OWNER" ]
  then
    echo "Applying: $SCRIPT as $OWNER"  
    echo "Applying: $SCRIPT as $OWNER" >> $LOG 
  else
    echo "Applying: $SCRIPT"  
    echo "Applying: $SCRIPT" >> $LOG  
  fi
  echo "Start Time:`date +'%H:%M'`"   
  echo "Start Time:`date +'%H:%M'`" >> $LOG  

  if [ ! -z "$OWNER" ]
  then
    if [ ! -z "$REAL_PASSWD" ]
    then
      PASSWD=$REAL_PASSWD
      # Verify the owner and get the password.
      TEST=`sqlplus -s '/ as sysdba' << ENDIT
set pause off head off pagesize 0 feedback off linesize 200
select username from dba_users where username = upper('$OWNER');
exit
ENDIT`
    else
      TEST=$OWNER
      # Verfiy the owner and get the password.
      PASSWD=`sqlplus -s '/ as sysdba' << ENDIT
set pause off head off pagesize 0 feedback off linesize 200
select password from dba_users where username = upper('$OWNER');
exit
ENDIT`
    fi
    if [ -z "$PASSWD" ] || [ ! "$TEST" = "$OWNER" ]
    then
      echo "**************************************************************************************** "
      echo "**************************************************************************************** " >> $LOG
      echo "WARNING: $PENDING specifies owner $OWNER for $SCRIPT but the owner does not exist in $ORACLE_SID." 
      echo "WARNING: $PENDING specifies owner $OWNER for $SCRIPT but the owner does not exist in $ORACLE_SID." >> $LOG
      echo "Skipping this script."
      echo "Skipping this script." >> $LOG
      echo "**************************************************************************************** "
      echo "**************************************************************************************** " >> $LOG
      echo "======================================================================================== " 
      echo "======================================================================================== " >> $LOG
      continue
    else
      # Write a script to grant DBA and switch to this schema owner.
      echo "grant DBA to $OWNER with admin option;" > $TEMPFILE1
      if [ ! -z "$REAL_PASSWD" ]
      then
        echo "connect $OWNER/${REAL_PASSWD}" >>$TEMPFILE1 
      else
        echo "alter user $OWNER identified by temp;" >> $TEMPFILE1
        echo "connect $OWNER/temp" >> $TEMPFILE1
        echo "alter user $OWNER identified by values '$PASSWD';" >> $TEMPFILE1
      fi
      # Write a post script to revoke the DBA privilege.
      echo "revoke DBA from $OWNER;" > $TEMPFILE2
    fi
  else
    # Null Script
    > $TEMPFILE1
    > $TEMPFILE2
  fi
 
  SCRIPT_LOG=${SID_DIR}/`echo $SCRIPT | cut -d. -f1`.log 
  >$SCRIPT_LOG

  # Check if any host commands are in the script that should be run first.
  if [ ! -z "`grep \"\-\- HOST COMMAND:\" ${DOCS}/${SRC_DIR}/${SCRIPT}`" ]
  then
    grep "\-\- HOST COMMAND:" ${DOCS}/${SRC_DIR}/${SCRIPT} | cut -c17- | tr -d '\r' | while read CMD
    do
      echo Running Host Command: $CMD 
      echo Running Host Command: $CMD >> $SCRIPT_LOG
      ${CMD} >>$SCRIPT_LOG 2>>$SCRIPT_LOG
    done
  fi

if [ "$RESTRICT_HOST_COMMAND" = "YES" ]
then
  RESTRICT_HOST="-r 1"
else
  RESTRICT_HOST=""
fi

sqlplus $RESTRICT_HOST /nolog << ENDIT >$SCRIPT_LOG 2>$SCRIPT_LOG
connect / as sysdba
set echo off feedback off
set sqlblanklines on
set define off
@$TEMPFILE1
set echo on feedback on term on scan off pagesize 500 linesize 120
show user
@${DOCS}/${SRC_DIR}/${SCRIPT}
set echo off feedback off
@$TEMPFILE2
exit
ENDIT
  echo "  End Time:`date +'%H:%M'`" 
  echo "  End Time:`date +'%H:%M'`" >> $LOG  
  echo "  Log file: $SCRIPT_LOG"
  echo "  Log file: $SCRIPT_LOG" >> $LOG
  if [ ! -z "`grep ORA- $SCRIPT_LOG`" ] || [ ! -z "`grep SP2- $SCRIPT_LOG`" ]
  then
    echo "************************************************************************************************** "
    echo "************************************************************************************************** " >> $LOG
    echo "WARNING: $SCRIPT_LOG contains errors:" 
    echo "WARNING: $SCRIPT_LOG contains errors:" >> $LOG
    egrep '(ORA-|SP2-)' ${SID_DIR}/$SCRIPT_LOG 
    egrep '(ORA-|SP2-)' ${SID_DIR}/$SCRIPT_LOG >> $LOG
    echo "************************************************************************************************** "
    echo "************************************************************************************************** " >> $LOG
    STATUS="with errors."
  else
    STATUS=""
  fi
  # Take the opertunity to convert the carriage returns to Intel format.
  awk 'sub("$", "\r")' $SCRIPT_LOG > $TEMPFILE1 
  cat $TEMPFILE1 > $SCRIPT_LOG
  echo " " 
  echo " " >> $LOG
  cat $SCRIPT_LOG 
  cat $SCRIPT_LOG >> $LOG
 
  # Append a seperation : onto the OWNER variable
  if [ ! -z "$OWNER" ]
  then
    if [ ! -z "${REAL_PASSWD}" ]
    then
      # Pad the OWNER variable with the REAL_PASSWD
      OWNER=`echo ${OWNER}/${REAL_PASSWD}:`
    else
      OWNER=`echo ${OWNER}:`
    fi 
  fi

  # If this is production then we comment the script out in the Pending list
  if [ "$PRODUCTION" = "YES" ]
  then
    echo "# ${OWNER}${SCRIPT} --Applied to Production $STATUS `date +'%b%d %I:%M%p'`" >> ${SID_DIR}/PENDING_IN_PROGRESS.txt
    echo ${OWNER}${SCRIPT} >> $SID_DIR/Latest_Applied.txt
  else
    echo "${OWNER}${SCRIPT} --Last Applied $STATUS `date +'%b%d %I:%M%p'`" >> ${SID_DIR}/PENDING_IN_PROGRESS.txt
  fi

done

# We move any script or log not in the PENDING list that is older than 6 weeks
# to the SID_DIR/OLD_SCRIPTS_LOGS directory if one exists.
if [ -d ${SID_DIR}/OLD_SCRIPTS_LOGS ]
then 
  for FILE in `ls -ltr ${SID_DIR}/${PROD_SID}* | egrep -i '(.sql$|.log)' | rev | cut -d\/ -f1 | rev | cut -d. -f1 | sort | uniq`
  do
    if [ -z "`grep $FILE ${SID_DIR}/${PENDING}`" ]
    then
      # 6 weeks old
      if [ ! -z "`find ${SID_DIR}/${FILE}* -atime +42`" ]
      then
        mv ${SID_DIR}/${FILE}* ${SID_DIR}/OLD_SCRIPTS_LOGS
      fi
    fi
  done
fi

if [ "$PRODUCTION" = "YES" ]
then
  # Comment out the scripts in $SID_DIR/Latest_Applied.txt in the other PENDING.txt files
  # Normally the Current SRC_DIR is the only PENDING.txt file that needs to be updated, but
  # and that might be the same as the current SID_DIR in which case the script has already
  # been commented out in the PENDING.txt file, but we will also check PENDING.txt
  # files in any existing Qual, Test, and Dev directories for this SID
  # Create as sed script from Latest_Applied.txt for making the changes.
  > $TEMPFILE1
  ADDED_TO_LOG=NO
  for SCRIPT in `cat $SID_DIR/Latest_Applied.txt | grep -v ^# | grep -v ^$`
  do
    echo "s/^$SCRIPT/# $SCRIPT --Applied to Production `date +'%b%d'`/g" >> $TEMPFILE1
  done

  for SRC_DIR in `echo $LOOK_IN_LIST`
  do
    DIR=/oracle_share/docs/${SRC_DIR}
    if [ ! "$DIR" = "$SID_DIR" ]
    then
      OTHER_PENDING=`ls ${DIR} | grep -i PENDING.txt$` 
      if [ ! -z "$OTHER_PENDING" ]
      then
        cat ${DIR}/$OTHER_PENDING > $TEMPFILE2
        sed -f $TEMPFILE1 $TEMPFILE2 > ${DIR}/$OTHER_PENDING
        if [ "${ADDED_TO_LOG}" = "NO" ]
        then
          ADDED_TO_LOG=YES
          echo "" >> $LOG
          echo "Revised PENDING list:"  
          echo "Revised PENDING list:" >> $LOG 
          cat ${DIR}/$OTHER_PENDING 
          cat ${DIR}/$OTHER_PENDING >> $LOG
          echo "" >> $LOG
          echo "" >> $LOG
        fi
      fi
    fi
  done
fi

# Replace PENDING with PENDING_IN_PROGRESS.txt
cat ${SID_DIR}/PENDING_IN_PROGRESS.txt > ${SID_DIR}/${PENDING}

# Update the Master Log
echo "======================================================================================" >> ${SID_DIR}/historical_pending.log
echo "======================================================================================" >> ${SID_DIR}/historical_pending.log
date >> ${SID_DIR}/historical_pending.log
echo "--------------------------------------------------------------------------------------" >> ${SID_DIR}/historical_pending.log
set echo off feedback off
cat ${SID_DIR}/PENDING_IN_PROGRESS.txt >> ${SID_DIR}/historical_pending.log

# Remove PENDING_IN_PROGRESS.txt
rm ${SID_DIR}/PENDING_IN_PROGRESS.txt


if [ ! -z "`echo $SID | grep TIPS `" ]
then
  echo "Updating grants to TIPS_READONLY & TIPS_UPDATE."
  echo "Updating grants to TIPS_READONLY & TIPS_UPDATE." >> $LOG
sqlplus /nolog << ENDIT >>$LOG 2>>$LOG
connect / as sysdba
set serveroutput on size 900000
exec sys.tips_grants_update_proc
exit
ENDIT
fi

# Run the checkup script.
echo "Running checkup.sh script on $SID."
echo "Running checkup.sh script on $SID." >> $LOG
$SCRIPT_DIR/checkup.sh $SID 

proc_cleanup
exit 0
# end of command file
