#!/bin/ksh
#
# File: apply_pending_check_in_box.sh

#   This script checks a database's script in-box folder for new scripts, copies them to docs folder
# reformats them as needed and applies the scripts to the test or quality database with a call to
# the apply_pending.sh script. 
#   Scripts are moved from the developers directory to the script_in_box directory on Unix by a batch file that 
# run on srvdba1oke which runs every 2 minutes and moves the files from their UNC path to the /oracle_share/docs path
# via Samba runing on lxtest04.

# Sample Batch File:
# date /T > QTIPS.log
# time /T >>QTIPS.log
# net use s: "\\lxtest04\oracle_docs" /USER:o_share o_share >>QTIPS.log
# dir \\srvapp525oke\data\qtips\support\deployment\QTIPS_script_drop >>QTIPS.log
# IF EXIST \\srvapp525oke\data\qtips\support\deployment\QTIPS_script_drop\PENDING.txt date /T > S:\RSTEST8_QTIPS\script_in_box\NEW_FILES_FLAG.txt
# move \\srvapp525oke\data\qtips\support\deployment\QTIPS_script_drop\PENDING.txt S:\RSTEST8_QTIPS\script_in_box >>QTIPS.log
# IF EXIST \\srvapp525oke\data\qtips\support\deployment\QTIPS_script_drop\*.sql date /T > S:\RSTEST8_QTIPS\script_in_box\NEW_FILES_FLAG.txt
# move \\srvapp525oke\data\qtips\support\deployment\QTIPS_script_drop\*.sql S:\RSTEST8_QTIPS\script_in_box >>QTIPS.log
# dir S:\RSTEST8_QTIPS\script_in_box >>QTIPS.log
 

# Author: Doug Jackson  5/20/2010.

#--------------------------------------------------------------------
#- Clean Up
#--------------------------------------------------------------------
proc_cleanup()
{
  # Check $LOG for ERROR and send mail if it exists.
  if [ ! -z "`grep ^ERROR: $LOG`" ]
  then
    SUBJECT="ERROR: apply_pending_check_in_box.sh for $SID"
    SEND=YES
  else
    if [ ! -z "`grep ^NOTE: $LOG`" ]
    then
      SUBJECT="NOTE: Pending list updated for $SID"
      SEND=YES
    else
      SEND=NO
    fi
  fi

  if [ "$SEND" = "YES" ]
  then

    # Send to recipients for ALL sids
    cat $EMAIL_LIST | grep -v "^#" | grep "^ALL:" | cut -d: -f2 >$TEMPFILE1
    # Send to recipients for current sid
    cat $EMAIL_LIST | grep -v "^#" | grep "^$SID:" | cut -d: -f2 >>$TEMPFILE1

    echo "" >>$LOG
    echo "Notified:" >>$LOG  
    for EACH in `cat $TEMPFILE1  | sort | uniq `
    do
      echo  $EACH >>$LOG
    done

    for EACH in `cat $TEMPFILE1  | sort | uniq `
    do
      mail -s "$SUBJECT" $EACH < $LOG
    done

  fi

  rm -f $TEMPFILE1 2>/dev/null
}

#################################################################################################################
### MAIN PROGRAM
#################################################################################################################

SCRIPT_DIR=`dirname $0`
if [ "$SCRIPT_DIR" = "." ]
then
  SCRIPT_DIR=`pwd`
fi
cd $SCRIPT_DIR

HOST=`hostname`
HOST=`echo $HOST | sed 's/.oneok.com//g'`

#Configuration
EMAIL_LIST="$SCRIPT_DIR/sid_email.txt"
REQUIRE_SCHEMA="DVLP:TEST:PROD DEV10:TST10:PRD10 DEV11:TST11:PRD11"
ORATAB=/etc/oratab
DOCS=/oracle_share/docs
TEMPFILE1=$SCRIPT_DIR/apply_pending_check_in_box_1_$$.tmp

if [ ! -f $EMAIL_LIST ]
then
  echo "ERROR: $EMAIL_LIST DOES NOT EXIST! apply_pending_check_in_box.sh failed!"
  mail -s "ERROR: $EMAIL_LIST DOES NOT EXIST! apply_pending_check_in_box.sh failed!" djackson@oneok.com < /dev/null
  exit 1
fi

if [ $# -lt 1 ]
then
  echo "Usage:  apply_pending_check_in_box.sh <Target_DB> [Schema] [/in_box_path/in_box_directory_name]"
  echo "  Target_DB - Database where scripts listed in /docs/HOST_Target_DB/PENDING.txt will be applied."
  echo "  The Target_DB must be on current Host."
  echo "  Source Directory Defaults, in order to: HOST_Target_DB, Quality, Test, Development."
  echo "  in_box default is scripts_in_box in the database docs directory." 
  exit 0
fi

# Check for oratab file existence
if [ ! -f $ORATAB ]
then
  echo "ERROR: $ORATAB DOES NOT EXIST! - apply_pending_check_in_box.sh failed!"
  exit 1
fi

SID=`echo $1 | tr [a-z] [A-Z]`

if [ ! -z "`echo $REQUIRE_SCHEMA | grep $SID`" ]
then
  # A schema is required
  if [ -z "$2" ]
  then
    echo "ERROR: The $SID database contains multiple schemas. A schema name must be specified"
    echo "Usage:  apply_pending.sh <Target_DB> [Schema]"
    exit 1
  fi
  SCHEMA=`echo $2 | tr [a-z] [A-Z]`
  LOG=$SCRIPT_DIR/apply_pending_check_in_box_${SID}_${SCHEMA}.log
  MASTER_LOG=$SCRIPT_DIR/apply_pending_check_in_box_${SID}_${SCHEMA}_master.log
  IN_BOX=$3
else
  SCHEMA=""
  LOG=$SCRIPT_DIR/apply_pending_check_in_box_${SID}.log
  MASTER_LOG=$SCRIPT_DIR/apply_pending_check_in_box_${SID}_master.log
  IN_BOX=$2
fi

echo "Apply Pending Check In-Box: $SID $SCHEMA  `date`"
echo "Apply Pending Check In-Box: $SID $SCHEMA  `date`" >$LOG

# Verify this SID is in the ORATAB
if [ -z "`grep ^${SID}: $ORATAB`" ]
then
  echo "ERROR: SID: $SID not found in oratab." >> $LOG
  proc_cleanup
  exit 1
fi

# Check if there is a clone running for this SID.
if [ ! -z "`ps -ef | grep clone_snap_ | grep _2_${SID}.sh`" ]
then
  echo "NOTE: A clone to $SID is running, so we do nothing until the clone completes." >> $LOG
  proc_cleanup
  exit 0
fi

# Verify SID_DIR
if [ ! -z "$SCHEMA" ]
then
  SID_DIR=$DOCS/`echo $HOST | tr [a-z] [A-Z]`_${SID}_${SCHEMA}
else
  SID_DIR=$DOCS/`echo $HOST | tr [a-z] [A-Z]`_${SID}
fi

if [ ! -r ${SID_DIR} ]
then
  echo "ERROR: Docs Directory: $SID_DIR was not found or is not readable." >> $LOG
  proc_cleanup
  exit 1
fi

# Check if there is a PENDING_IN_PROGRESS.txt file
if [ -f "${SID_DIR}/PENDING_IN_PROGRESS.txt" ]
then
  echo "PENDING_IN_PROGRESS.txt file was found in $SID_DIR, so we do nothing until it completes." >> $LOG
  proc_cleanup
  exit 0
fi

# Verify IN_BOX
if [ -z "$IN_BOX" ]
then
  # Default
  IN_BOX=${SID_DIR}/script_in_box 
fi

if [ ! -r $IN_BOX ]
then
  echo "ERROR: Script in-box: $IN_BOX is not readable or does not exists." >$LOG
  proc_cleanup
  exit 1
fi

# Check if there is anything in the IN_BOX
if [ -z "`ls $IN_BOX | egrep -i '(.sql$|pending.txt$)'`" ]
then
  echo "The in-box contains no pending.txt or sql scripts." >>$LOG
  proc_cleanup
  exit 0
fi

# Check that if there is a NEW_FILES_FLAG.txt file, that it is at least 5 minutes old.
if [ -f $IN_BOX/NEW_FILES_FLAG.txt ]
then
  if [ ! -z "`find $IN_BOX -name "NEW_FILES_FLAG.txt" -mmin -5`" ]
  then
    echo "There are files in the in-box but they are not yet 5 minutes old."
    proc_cleanup
    exit 0
  else
    rm $IN_BOX/NEW_FILES_FLAG.txt
  fi
fi

# Check if there is a PENDING.txt file
PENDING=`ls ${SID_DIR} | grep -i ^PENDING.txt$`
if [ -z "$PENDING" ]
then
  # No PENDING.txt found so we create one.
  echo "# PENDING" >${SID_DIR}/PENDING.txt
  PENDING=PENDING.txt
fi

# Check for and updated Pending list in the IN_BOX
NEW_PENDING=`ls ${IN_BOX} | grep -i ^PENDING.txt$`
if [ ! -z "$NEW_PENDING" ]
then
  echo "" >>$LOG
  echo "=====================================================================================" >> $MASTER_LOG
  date >> $MASTER_LOG 
  echo "NOTE: Updated Pending List." >>$LOG
  echo "NOTE: Updated Pending List." >>$MASTER_LOG
  echo "NEW PENDING LIST" >>$LOG
  echo "NEW PENDING LIST" >>$MASTER_LOG
  cat  ${IN_BOX}/$NEW_PENDING | grep -v "^$" >>$LOG
  cat  ${IN_BOX}/$NEW_PENDING | grep -v "^$" >>$MASTER_LOG
  echo "" >>$LOG
  echo "" >>$MASTER_LOG
  # Strip out blank lines and reformat it for UNIX
  echo "# PENDING SCRIPTS:" > $TEMPFILE1
  cat ${IN_BOX}/$NEW_PENDING | grep -v "^$" | grep -vi PENDING | tr -d '\r' | tr -d '\015\032'  >> $TEMPFILE1
  # Add a trailing blank line
  echo "" >> $TEMPFILE1
  echo "" >>$LOG
  echo "DIFFERENCES:" >>$LOG
  echo "DIFFERENCES:" >>$MASTER_LOG
  diff $TEMPFILE1 ${SID_DIR}/$PENDING >>$LOG
  diff $TEMPFILE1 ${SID_DIR}/$PENDING >>$MASTER_LOG
  cat $TEMPFILE1 > ${SID_DIR}/$PENDING
  rm -f ${IN_BOX}/$NEW_PENDING
  echo "" >>$LOG
  echo "" >>$MASTER_LOG
fi  

# Check if this is a production host
if [ ! -z "`echo $HOST | grep -i prod`" ]
then
  PRODUCTION="YES"
  PROD_SID=$SID
else
  PRODUCTION="NO"
  # Trim off the fist character to get the production database name
  PROD_SID=`echo $SID | cut -c 2-`
fi
if [ ! -z "$SCHEMA" ]
then
  for SET in `echo $REQUIRE_SCHEMA`
  do
    if [ ! -z "`echo $SET | grep $SID`" ]
    then
      PROD_SID=`echo $SET | cut -d: -f3`
      TEST_SID=`echo $SET | cut -d: -f2`
      DVLP_SID=`echo $SET | cut -d: -f1`
    fi
  done
fi

if [ -z "`find $IN_BOX | grep -i .sql$`" ]
then
  echo "No scripts to apply." >>$LOG
else
  echo "" >>$LOG
  echo "Following scripts were added to the pending list from the in-box:" >>$LOG
  # Check the IN_BOX for scripts.
  find $IN_BOX | grep -i .sql$ | sort | while read FILE
  do
    FILE=`echo ${FILE} | rev | cut -d/ -f1 | rev`
    if [ ! -z "`echo $FILE | grep \" \" `" ]
    then
      NEW_FILE=`echo $FILE | sed 's/ /\_/g' | sed 's/\_\_/\_/g'`
    else
      NEW_FILE=$FILE
    fi
    if [ ! "`echo $FILE | cut -d'_' -f1`" = "${PROD_SID}" ]
    then 
      if [ ! -z "$SCHEMA" ]
      then
        NEW_FILE=${PROD_SID}_${SCHEMA}_`date +"%b%d"`_${NEW_FILE}
      else 
        NEW_FILE=${PROD_SID}_`date +"%b%d"`_${NEW_FILE} 
      fi
    fi

    # Move the file from the IN_BOX to the SCRIPT_DIR
    mv "${IN_BOX}/${FILE}" ${SID_DIR}/${NEW_FILE}
   
    if [ ! -z "$SCHEMA" ]
    then
      echo "/* current_schema command added by apply_pending_check_in_box.sh */" > $TEMPFILE1 
      echo "alter session set current_schema=${SCHEMA};" >> $TEMPFILE1 
      cat ${SID_DIR}/${NEW_FILE} >> $TEMPFILE1
      cat $TEMPFILE1 > ${SID_DIR}/${NEW_FILE}
    fi
    # Append the NEW_FILE to the PENDING list.
    echo ${NEW_FILE} >> ${SID_DIR}/${PENDING}
    echo ${NEW_FILE} >> $LOG
  done 

  # Call apply_pending.sh
  ${SCRIPT_DIR}/apply_pending.sh $SID $SCHEMA -new
fi

proc_cleanup
exit 0
# end of command file
